<?php
/**
 * BloggerAI for Ai Blogging
 *
 * @package     BloggerAI for Ai Blogging
 * @author      Paolo De Giglio
 * @copyright   2023
 * @license     GPL-2.0-or-later
 *
 * @wordpress-plugin
 * Plugin Name: BloggerAI for Ai Blogging
 * Plugin URI:  https://wordpress.org/plugins/bloggerai-for-ai-blogging 
 * Description: This plugin permits the connection between your WordPress site to BloggerAI's services.
 * Version:     1.0.0
 * Author:      Paolo De Giglio 
 * Author URI:  https://bloggerai.app
 * Text Domain: bloggerai
 * License:     GPL v2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 */

// Add as permalink /bloggerai/v1/post

// If the wordpress permalink is set to plain 
// add the following line to the .htaccess file

add_action( 'init', function () {
    register_rest_route( 'bloggerai/v1', '/post', array(
        'methods' => 'POST',
        'callback' => 'bloggerai_post',
   ) );
} );

/**
 * Grab latest post title by an author!
 *
 * @param array $data Options for the function.
 * @return string|null Post title for the latest, * or null if none.
 */
function bloggerai_post( $data ) {
    // Get the title from the post body
    $title = sanitize_text_field($data['title']); 
    $content = wp_kses_post($data['content']); 
    $status = sanitize_text_field($data['status']); 
    $id = sanitize_text_field($data['id']); 
    
    // Validate the title, content, status and id
    if($title == '' || $content == '' || $status == '' || $id == '') {
        return new WP_Error( 'bloggerai_internal_error', 'Title, content, status or id is not set', array( 'status' => 400 ) );
    }
    
    // Escape the title, content, status and id
    $title = esc_textarea($title);
    $status = esc_textarea($status);
    $id = esc_textarea($id);


    // Get the API key from the header of the request
    $headers = apache_request_headers();
    $api_key = $headers['Authorization'];

    // Check if the API key is correct
    $currentApiKey = esc_textarea(get_option('bloggerai_api_key_stored'));

    // If the current API key is not set or is shorter than 10 characters return error
    if($currentApiKey == '' || strlen($currentApiKey) < 10) {
        return new WP_Error( 'bloggerai_api_key_error', 'API key is not correct', array( 'status' => 401 ) ); 
    }

    try {
        if($api_key != $currentApiKey || !isset($currentApiKey)) {
            // Return response error with status code 401
            return new WP_Error( 'bloggerai_api_key_error', 'API key is not correct', array( 'status' => 401 ) ); 
        } else {
            
            // If title is not set return error
            if($title == '') {
                return new WP_Error( 'bloggerai_title_error', 'Title is not set', array( 'status' => 400 ) ); 
            }
            if($content == '') {
                return new WP_Error( 'bloggerai_content_error', 'Content is not set', array( 'status' => 400 ) ); 
            }


            // Create post object
            $my_post = array(
                'post_title'    => $title,
                'post_content'  => $content,
                'meta_input'    => array(
                    'bloggerai_id' => $id
                ),
            ); 

            // Search into the meta_input if the post already exist
            $args = array(
                'meta_key' => 'bloggerai_id',
                'meta_value' => $id,
                'post_type' => 'post',
                'post_status' => 'any',
                'posts_per_page' => 1
            );
            $query = new WP_Query( $args );
            $postExists = false;
            if ( $query->have_posts() ) {
                $my_post['ID'] = $query->post->ID;
                $postExists = true;
            }

            // Look if the post exist by looking for the same ID
            if(!$postExists){
                // Insert the post into the database
                $result= wp_insert_post( $my_post, true );
                if ( is_wp_error( $result ) ) {
                    return new WP_Error( 'bloggerai_internal_error', $result->get_error_message(), array( 'status' => 500 ) );
                }
            } else {
                // Update the post with the same id into the database 
                $result= wp_update_post( $my_post, true );
                if ( is_wp_error( $result ) ) {
                    return new WP_Error( 'bloggerai_internal_error', $result->get_error_message(), array( 'status' => 500 ) );
                } 
            }

            // Return response success with status code 200
            return new WP_REST_Response( 'Post created', 200 );
        }     
    } catch (\Throwable $th) {
        // Return response error with status code 500 and the error message
        return new WP_Error( 'bloggerai_internal_error', $th->getMessage(), array( 'status' => 500 ) );
    }

}

// Add a section in wich there is an input field for the API key
add_action( 'admin_menu', 'bloggerai_menu' );
function bloggerai_menu() {
    add_options_page( 'BloggerAI', 'BloggerAI', 'manage_options', 'bloggerai', 'bloggerai_options' );
}

function bloggerai_options() {
    if ( !current_user_can( 'manage_options' ) )  {
        wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }
    echo '<div class="bloggerai-css-wrap">';
    echo '<h2>BloggerAI</h2>';
    // Add the API key input field
    echo '<p class="bloggerai-css-paragraph">To connect your WordPress site to BloggerAI, you\'ll need an API key. 
    This key is unique to your account and will allow you to securely authenticate your website with BloggerAI\'s servers. 
    Once you have your API key, you\'ll be able to easily publish blog posts from BloggerAI directly to your WordPress site. 
    To generate your key just click the button below, remember to copy and save in a secure place.</p>';
    echo '<div id="flexboxForInput">';
    echo '<input type="text" id="bloggerai_api_key_stored" disabled name="bloggerai_api_key_stored" value="' . esc_textarea(get_option('bloggerai_api_key_stored')) . '">';
    // Add button to random generate the API key
    echo '<button id="bloggerai_generate_api_key">Generate</button>';
    // Add button to copy the API key
    echo '<button id="bloggerai_copy_api_key">Copy</button>';
    // Add button to save the API key
    echo '</div>';
    // If the API key is set show that the plugin is connected
    if (get_option('bloggerai_api_key_stored')) {
        echo '<p id="connectionDone" style="display:block">Now you need to copy the above key into the BloggerAI publishment settings</p>';
    } else {
        echo '<p id="connectionDone" style="display:none">Now you need to copy the above key into the BloggerAI publishment settings</p>';
    }

    echo '<div id="myTooltip"></div>';

    echo '</div>';
    // Save the API key
    echo '<script>
    button = document.getElementById("bloggerai_generate_api_key");
    button.addEventListener("click", function() {
        var key = Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
        document.getElementById("bloggerai_api_key_stored").value = key;
        var key = document.getElementById("bloggerai_api_key_stored").value;
        var data = {
            "action": "bloggerai_save_api_key",
            "key": key
        };
        jQuery.post(ajaxurl, data, function(response) {
            document.getElementById("connectionDone").style.display = "block";
        });
    });
    button = document.getElementById("bloggerai_copy_api_key");
    button.addEventListener("click", function() {
        var copyText = document.getElementById("bloggerai_api_key_stored");
        var value = copyText.value;
        
        // Create a "hidden" input
        var aux = document.createElement("input");
        // Assign it the value of the specified element
        aux.setAttribute("value", value);
        // Append it to the body
        document.body.appendChild(aux);
        // Highlight its content
        aux.select();
        // Copy the highlighted text
        document.execCommand("copy");
        // Remove it from the body
        document.body.removeChild(aux);

        var tooltip = document.getElementById("myTooltip"); 
        tooltip.innerHTML = "Copied: " + copyText.value;
    });
    </script>';
    echo '<style>
    .bloggerai-css-paragraph {
        font-size: 1.2rem;
        margin-bottom: 20px;
    }
    .bloggerai-css-wrap {
        width: 80%;
        margin: 0 auto;
        padding: 40px 0;

        h2 {
            font-size: 2rem;
        }

        p {
            font-size: 1.5rem;
        }
    }
    .bloggerai-css-logo-img {
        height: 60px;
        margin-bottom: 20px;
        margin-top: 20px;
        display: block;
        margin: 0 auto;
    }
    #bloggerai_api_key_stored {
        width: 100%;
        padding: 12px 20px;
        display: inline-block;
        border: 1px solid #ccc;
        border-radius: 4px;
        box-sizing: border-box;
    }
    #bloggerai_generate_api_key {
        background-color: #D64550;
        border: none;
        color: white;
        padding: 12px 32px;
        text-align: center;
        text-decoration: none;
        display: inline-block;
        font-size: 16px;
        cursor: pointer;
    }
    #bloggerai_copy_api_key {
        background-color: #EB8850;
        border: none;
        color: white;
        padding: 12px 32px;
        text-align: center;
        text-decoration: none;
        display: inline-block;
        font-size: 16px;
        cursor: pointer;
    }
    #myTooltip {
        position: relative;
        display: inline-block;
        border-bottom: 1px dotted black;
        transition: opacity 1s;
    }
    #flexboxForInput {
        display: flex;
        flex-direction: row;
        justify-content: space-between;
        gap: 10px;
        margin-top: 20px;
        margin-bottom: 20px;

        input {
            width: 70%;
        }
    }
    </style>';

    }

// Save the API key
add_action( 'wp_ajax_bloggerai_save_api_key', 'bloggerai_save_api_key' );
function bloggerai_save_api_key() {
    $key = sanitize_text_field($_POST['key']); 
    // Validate the API key
    if (strlen($key) < 8) {
        die();
    }
    // Escape the API key
    $key = esc_textarea($key);
    update_option('bloggerai_api_key_stored', $key);
    die();
}
